import matplotlib.pyplot as plt
import numpy as np
import time

# from newton import ...

def compute_point(c, num_iterations):
    """
    Compute the value of a point in the fractal
    """

    z = c
    for j in range(num_iterations):
        dz = (z*z*z - 1)/(3*z*z)

        # Other formulas:
        #
        #     dz = (z*z*z - 2*z + 2) / (3*z*z - 2)
        #     dz = (z*z*z*z*z - 3j*z*z*z - (5 + 2j)) / (5*z*z*z*z - 9j*z*z)
        #
        # Or invent your own: in general
        #
        #     dz = p(z) / p'(z)
        #
        # where p(z) is some polynomial (or other function) of z and
        # p'(z) is its derivative.
        #
        
        z -= dz
        if abs(dz) < 1e-3:
            break
    else:
        z = -99
    return z

def compute_newton_fractal(x_min, x_max, y_min, y_max, nx, ny, num_iterations):
    """
    Compute the Newton fractal
    """

    result = np.zeros((nx, ny), complex)

    dx = (x_max - x_min) / float(nx - 1)
    dy = (y_max - y_min) / float(ny - 1)
    
    for i in range(result.shape[0]):
        for j in range(result.shape[1]):
            c = (x_min + dx*i) + (y_min + dy*j)*1j
            result[i, j] = compute_point(c, num_iterations)

    return result

def main():
    """
    Plot the Newton fractal
    """

    fractal = compute_newton_fractal(x_min=-3, x_max=3,
                                     y_min=-3, y_max=3,
                                     nx=500, ny=500,
                                     num_iterations=50)

    fractal[fractal == -99] = np.nan
    plt.imshow(np.arctan2(fractal.real, fractal.imag))
    plt.show()

if __name__ == "__main__":
    # This is the 'standard' way of using a 'main' function as the entry
    # point to the program in Python
    main()
