/*
    Anagrammimylly: an anagram generator
    Copyright (C) 1998-2000 Pauli Virtanen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * anagram_find: Heart of the whole program. Code is working, quite clean and
 *               original ;)
 */
#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "wordlist.h"
#include "anagram.h"
#include "anagram_internal.h"

#define MAX_COMPOSED 500
#define MAX_WORD_LEN 500

typedef struct char_count_t
{
        char ch;
        int cnt;
} char_count_t;

typedef struct fast_word_t
{
        word_t *word;
        char_count_t *counts;
        int cnt;
} fast_word_t;

typedef struct anagram_find_t
{
        char *composed[MAX_COMPOSED];
        int notused;

        fast_word_t *fast_words;
        fast_word_t *fast_words_end;

        int net[256];

        fast_word_t **len_hash;

        outfunc_t *outinfo;
        outfunc_t *outlist;
        void *user_data;
} anagram_find_t;

/*
 * Checks whether wd1 contains the letters of wd2
 */
static char word_fits(int *net, char *wd2)
{
	char *pt;
	int cnt;

	while (*wd2 != '\0')
	{
                cnt = 1;
                for (pt = wd2+1; *pt != '\0'; ++pt)
                        if (*pt == *wd2) ++cnt;

                if (net[(unsigned char)*wd2] < cnt) return 0;
 		wd2++;
	}
	return 1;
}

static void use_net(int *net, char *wd)
{
        while (*wd != '\0')
        {
                --net[(unsigned char)*wd];
                ++wd;
        }
}
static void unuse_net(int *net, char *wd)
{
        while (*wd != '\0')
        {
                ++net[(unsigned char)*wd];
                ++wd;
        }
}

/*
 * Create fast_word_t for a word
 */
static char fast_fits(int *net, fast_word_t *lw)
{
        int c;
        for (c = 0; c < lw->cnt; ++c)
                if (net[(unsigned char)lw->counts[c].ch] < lw->counts[c].cnt)
                        return 0;
        return 1;
}
static void use_fast_net(int *net, fast_word_t *lw)
{
        int c;
        for (c = 0; c < lw->cnt; ++c)
                net[(unsigned char)lw->counts[c].ch] -= lw->counts[c].cnt;
}
static void unuse_fast_net(int *net, fast_word_t *lw)
{
        int c;
        for (c = 0; c < lw->cnt; ++c)
                net[(unsigned char)lw->counts[c].ch] += lw->counts[c].cnt;
}


static int *xxx_net;

static int main_net_sort(const void *a, const void *b)
{
        if (xxx_net[*((unsigned char*)a)] < xxx_net[*((unsigned char*)b)])
                return -1;
        else if (xxx_net[*((unsigned char*)a)] > xxx_net[*((unsigned char*)b)])
                return 1;
        else
                return 0;
}

static void init_fast_word(fast_word_t *lw, int *main_net, word_t *wd)
{
        int net[256];
        int c, cnt;

        lw->word = wd;

        memset(net, 0, 256*sizeof(int));
        unuse_net(net, wd->word);

        for (c = 0, cnt = 0; c < 256; ++c)
                if (net[c] > 0)
                        ++cnt;

        if (cnt == 0)
        {
                lw->cnt = 0;
                lw->counts = NULL;
                return;
        }

        lw->cnt = cnt;
        lw->counts = (char_count_t*)malloc(cnt * sizeof(char_count_t));
        cnt = 0;

        for (c = 0; c < 256; ++c)
                if (net[c] > 0)
                {
                        lw->counts[cnt].ch = (char)c;
                        lw->counts[cnt].cnt = net[c];
                        ++cnt;
                }

        xxx_net = main_net;
        qsort(lw->counts, cnt, sizeof(char_count_t), main_net_sort);
        return;
}
static void free_fast_word(fast_word_t *lw)
{
        if (lw->counts != NULL)
                free(lw->counts);
        lw->counts = NULL;
}


static success_t find_wl_recurse(anagram_find_t *d, int pos,
                                 fast_word_t *last)
{
	unsigned long idx;
        int wordlen;
        fast_word_t *fast_word;

        if (pos+1 >= MAX_COMPOSED) return failure; // liian pitk sana!
	
        fast_word = d->len_hash[d->notused];
	for (; fast_word < last; ++fast_word)
	{
		if (fast_fits(d->net, fast_word))
		{
			// notify user
                        if (pos < 1 && d->outinfo != NULL &&
                            (*d->outinfo)(fast_word->word->word, d->user_data)
                            != success)
                                return failure;

                        wordlen = fast_word->word->end - fast_word->word->word;
			d->composed[pos] = fast_word->word->word;
			d->composed[pos+1] = NULL;
			d->notused -= wordlen;
			use_fast_net(d->net, fast_word);

			if (find_wl_recurse(d, pos+1, fast_word) != success)
                                return failure;

			unuse_fast_net(d->net, fast_word);
			d->notused += wordlen;
			d->composed[pos] = NULL;
		}
	}

	if (d->notused == 0 && d->composed[0] != NULL) // Sana valmis
	{
                char buffer[MAX_WORD_LEN*2+100];
                strcpy(buffer, d->composed[0]);
		idx = 1;
		while (idx < MAX_COMPOSED && d->composed[idx] != NULL)
		{
			strcat(buffer, " ");
			strcat(buffer, d->composed[idx]);
			idx++;
		}
		if (d->outlist != NULL && (*d->outlist)(buffer, d->user_data) != success)
                        return failure;
	}
	return success;
}

static void find_wl_uninitialize(anagram_find_t *d)
{
	if (d->fast_words != NULL)
        {
                fast_word_t *w;
                for (w = d->fast_words; w != d->fast_words_end; ++w)
                        free_fast_word(w);
                free(d->fast_words);
                d->fast_words = NULL;
        }
        d->fast_words_end = NULL;
        if (d->len_hash != NULL)
        {
                free(d->len_hash);
                d->len_hash = NULL;
        }
}


static int fast_word_t_reverse_length_sort(const void *av, const void *bv)
{
        fast_word_t *a = (fast_word_t*)av;
        fast_word_t *b = (fast_word_t*)bv;
        return (a->word->end - a->word->word) - (b->word->end - b->word->word);
}

static success_t find_wl_initialize(anagram_find_t *d, char *word,
                                    wordlist_t *wordlist)
{
        word_t *w;
	unsigned long cnt = 0;
        unsigned long longest = strlen(word);

        d->len_hash = NULL;
        d->fast_words = NULL;
        d->fast_words_end = NULL;

        for (w = wordlist->begin; w != wordlist->end; ++w)
		if (word_fits(d->net, w->word))
                {
                        if (w->end - w->word > longest)
                                longest = w->end - w->word;
                        cnt++;
                }

        d->fast_words = (fast_word_t*)malloc(cnt * sizeof(fast_word_t));
	if (d->fast_words == NULL)
                return failure;
        d->len_hash = (fast_word_t**)malloc((longest+1)*sizeof(fast_word_t*));
	if (d->len_hash == NULL)
                return failure;
        

        for (w = wordlist->begin, cnt = 0; w != wordlist->end; ++w)
		if (word_fits(d->net, w->word))
		{
                        init_fast_word(d->fast_words + cnt, d->net, w);
			cnt++;
		}
        d->fast_words_end = d->fast_words + cnt;

        // sort according to length, longest words first
        qsort(d->fast_words, cnt, sizeof(fast_word_t),
              fast_word_t_reverse_length_sort);

        // initialize length hash
        {
                fast_word_t *f;
                unsigned long len = longest;

                d->len_hash[len] = d->fast_words;
                for (f = d->fast_words; f != d->fast_words_end; ++f)
                {
                        while (len > f->word->end - f->word->word)
                        {
                                --len;
                                d->len_hash[len] = f;
                        }
                }
                while (len > 0)
                {
                        --len;
                        d->len_hash[len] = d->fast_words_end;
                }
        }
	return success;
}

success_t find_anagrams(char *word, char *must_have,
                        wordlist_t *wordlist, outfunc_t *tolist,
                        outfunc_t *toinfo, void *user_data)
{
	char *pt, *pt2;
        anagram_find_t data;

	/* Make a copy of word */
	pt = strdup(word); word = pt;

	/* Remove non-letters from word and count the letters */
	for (pt = word, pt2 = word; *pt != '\0'; pt++) {
		if (my_isalpha(*pt)) {
			*pt2 = *pt;
			pt2++;
		}
	}
	*pt2 = '\0'; /* Terminate */

	/* Initialize data */
	data.outinfo = toinfo;
	data.outlist = tolist;
        data.user_data = user_data;

        memset(data.net, 0, 256*sizeof(int));

        unuse_net(data.net, word);

        data.fast_words = NULL;
        data.user_data = user_data;

	memset(data.composed, 0, sizeof(word_t*)*MAX_COMPOSED);

	data.notused = strlen(word);

        /* Remove the letters in the must_have word from the net */
        if (must_have)
        {
                for (pt = must_have; *must_have != '\0'; ++must_have)
                {
                        if (my_isalpha(*pt) &&
                            --data.net[(unsigned char)*must_have] < 0)
                        {
                                /* The must_have word didn't fit.
                                 * So there is no need to look further.
                                 */
				free(word);
                                return success;
                        }
                        else
                        {
                                data.notused--;
                        }
                }
                data.composed[0] = must_have;
        }

        if (find_wl_initialize(&data, word, wordlist) == success)
	{
                if (must_have)
                        find_wl_recurse(&data, 1, data.fast_words_end);
                else
                        find_wl_recurse(&data, 0, data.fast_words_end);
		find_wl_uninitialize(&data);
		free(word);
                return success;
	}
	free(word);
        return failure;
}
