/*
    Anagrammimylly: an anagram generator
    Copyright (C) 1998-2000 Pauli Virtanen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * Interface to the anagrammimylly library
 * (actually it should be merged into the library itself...)
 */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "anagram.h"
#include "anagram_internal.h"

typedef struct  {
    int index;
    int target;
    char *output;
} simple_anagram_t;

success_t simple_anagram_callback(char *word, void *v_data)
{
    simple_anagram_t *data = (simple_anagram_t*)v_data;
    if (data->index >= data->target) {
        data->output = strdup(word);
        return failure;
    }
    else {
        data->index += 1;
        return success;
    }
}

char *simple_anagram(char *dictfile,
                     char *word,
                     int index)
{
    char *lower_word;
    anagram_data_t data;
    simple_anagram_t simple_data;

    lower_word = strdup(word);
    str_tolower(word);
    
    anagram_init(&data);
    
    if (!anagram_wl_load(dictfile, &data))
    {
        return NULL;
    }

    simple_data.index = 0;
    simple_data.target = index;
    simple_data.output = NULL;

    anagram_set_output_functions(simple_anagram_callback,
                                 NULL,
                                 (void*)&simple_data,
                                 &data);
    anagram_find_word(lower_word, NULL, &data);
    anagram_uninit(&data);
    free(lower_word);

    return simple_data.output;
}

void simple_anagram_free(char *data)
{
    free(data);
}

success_t anagram_init(anagram_data_t *data)
{
        memset(data, 0, sizeof(anagram_data_t));
        return success;
}

success_t anagram_set_output_functions(outfunc_t *outlist, outfunc_t *outinfo,
                                       void *user_data, anagram_data_t *data)
{
        data->user_data = user_data;
        data->output_word = outlist;
        data->output_info = outinfo;
        return success;
}

success_t anagram_wl_load(char *wordlist, anagram_data_t *data)
{
        if (data->wl_loaded) { wl_unload(&data->wl); data->wl_loaded = 0; }
        if (wl_load(&data->wl, wordlist))
        {
                data->wl_loaded = 1;
                return success;
        }
        else return failure;
}

success_t anagram_wl_save(char *wordlist, anagram_data_t *data)
{
        if (!data->wl_loaded) return failure;
        return wl_save(&data->wl, wordlist);
}

success_t anagram_wl_add_word(char *word, anagram_data_t *data)
{
        char a_word[256];

        data->wl_loaded = 1;
    
        strcpy(a_word, word);
    
        str_tolower(a_word);
    
        if (wl_find_word(&data->wl, a_word) == data->wl.end)
        {
                if (strlen(a_word) > 1)
                        wl_add_word(&data->wl, a_word);
                return success;
        }
        else return failure;
}

success_t anagram_wl_remove_word(char *word, anagram_data_t *data)
{
        if (!data->wl_loaded) return failure;
        return wl_remove(&data->wl, wl_find_word(&data->wl, word));
}

success_t anagram_wl_add_file(char *file, anagram_data_t *data)
{
        data->wl_loaded = 1;
        return wl_add_file(file, &data->wl, data->output_info, data->user_data);
}

unsigned long anagram_wl_get_count(anagram_data_t *data)
{
        if (!data->wl_loaded) return 0;
        return wl_get_count(&data->wl);
}

unsigned long  anagram_wl_fix(anagram_data_t *data, char dup, char low,
                                    char one, char two, char sort,
                                    char remlong)
{
        if (!data->wl_loaded) return failure;
        return wl_fix(&data->wl, data->output_info, data->user_data,
                       dup, low, one, two, sort, remlong);
}

success_t anagram_find_word(char *word, char *must_have, anagram_data_t *data)
{
        if (!data->wl_loaded) return failure;
        return find_anagrams(word, must_have, &data->wl, data->output_word,
                             data->output_info, data->user_data);
}

success_t anagram_uninit(anagram_data_t *data)
{
        if (data->wl_loaded)
        {
                wl_unload(&data->wl);
                data->wl_loaded = 0;
        }
        return success;
}


