/*
    Anagrammimylly: an anagram generator
    Copyright (C) 1998-2000 Pauli Virtanen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*
 * lex_add: adding words from file to simple wordlist
 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "wordlist.h"
#include "anagram.h"
#include "anagram_internal.h"

success_t wl_add_file(char *file, wordlist_t *wordlist, outfunc_t *outinfo, void *user_data)
{
	FILE *infile = fopen(file, "rt");
	char line[2000];
	char word[2000];
	char *wordpt, *outpt;
	
	long filelen;
	long curpos;
	char percent[10];
	
	if (!infile) return failure;

	// Get the file's length
	fseek(infile, 0, SEEK_END);
	filelen = ftell(infile);
	fseek(infile, 0, SEEK_SET);
	strcpy(percent, "0%");

	while (fgets(line, 2000, infile))
	{
		if (outinfo)
		{
			curpos = ftell(infile);
			if (curpos > 0 && filelen > 0) sprintf(percent, "%i%%", (int)((curpos*100)/filelen));
			if ((*outinfo)(percent, user_data) != success) break;
		}
		
                wordpt=line;
		while (*wordpt != '\0')
		{
			// Skip over whitespace
			while (*wordpt != '\0' && !my_isalpha(*wordpt)) wordpt++;

			// If a letter here
			if (*wordpt != '\0')
			{
				// copy until string end or not a letter
				outpt = word;
				while (*wordpt != '\0' && my_isalpha(*wordpt))
					*(outpt++) = *(wordpt++);
				*outpt = '\0';
				
				// If word is followed by hyphen, then it is not worthy to be added:
				// it might be hyphenated and therefore partial.
				if (*wordpt != '-')
				{
					// If word is long enough and not yet in there
					// then put it there in lowercase
					str_tolower(word);
					if (strlen(word) > 1 &&
                                            wl_find_word(wordlist, word) !=
                                            wordlist->end)
						wl_add_word(wordlist, word);
				}
			}
		}
	}

	fclose(infile);
	return success;
}



